@@ -12,59 +12,74 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
 
-import sys
+from __future__ import annotations
+
+from typing import Any
+from typing import Optional
+
 import logging
 
 import nbxmpp
 from nbxmpp.client import Client as NBXMPPClient
+from nbxmpp.protocol import JID
 from nbxmpp.const import StreamError
 from nbxmpp.const import ConnectionType
 
 from gi.repository import GLib
-
-from gajim.common import passwords
-from gajim.common.nec import NetworkEvent
+from gi.repository import Gio
+from gi.repository import GObject
+from gi.repository import Gtk
 
 from gajim.common import app
 from gajim.common import helpers
 from gajim.common import modules
+from gajim.common import passwords
+from gajim.common.events import AccountConnected
+from gajim.common.events import MessageSent
+from gajim.common.events import AccountDisonnected
+from gajim.common.events import PasswordRequired
+from gajim.common.events import PlainConnection
+from gajim.common.events import SignedIn
+from gajim.common.events import Notification
+from gajim.common.events import StanzaReceived
+from gajim.common.events import StanzaSent
 from gajim.common.const import ClientState
-from gajim.common.helpers import get_encryption_method
+from gajim.common.const import SimpleClientState
 from gajim.common.helpers import get_custom_host
-from gajim.common.helpers import get_user_proxy
+from gajim.common.helpers import get_account_proxy
 from gajim.common.helpers import warn_about_plain_connection
 from gajim.common.helpers import get_resource
-from gajim.common.helpers import get_ignored_ssl_errors
 from gajim.common.helpers import get_idle_status_message
+from gajim.common.helpers import Observable
 from gajim.common.idle import Monitor
+from gajim.common.idle import IdleMonitorManager
 from gajim.common.i18n import _
+from gajim.common.structs import OutgoingMessage
 
-from gajim.common.connection_handlers import ConnectionHandlers
-from gajim.common.connection_handlers_events import MessageSentEvent
-
-from gajim.gtk.util import open_window
+from gajim.gui.util import open_window
 
 
 log = logging.getLogger('gajim.client')
 
 
-class Client(ConnectionHandlers):
-    def __init__(self, account):
+IgnoredTlsErrorsT = Optional[set[Gio.TlsCertificateFlags]]
+
+
+class Client(Observable):
+    def __init__(self, account: str) -> None:
+        Observable.__init__(self, log)
         self._client = None
         self._account = account
         self.name = account
-        self._hostname = app.config.get_per(
-            'accounts', self._account, 'hostname')
-        self._user = app.config.get_per('accounts', self._account, 'name')
+        self._hostname = app.settings.get_account_setting(self._account,
+                                                          'hostname')
+        self._user = app.settings.get_account_setting(self._account, 'name')
         self.password = None
 
         self._priority = 0
         self._connect_machine_calls = 0
-        self.avatar_conversion = False
         self.addressing_supported = False
 
-        self.is_zeroconf = False
-        self.pep = {}
         self.roster_supported = True
 
         self._state = ClientState.DISCONNECTED
@@ -80,8 +95,6 @@ class Client(ConnectionHandlers):
         self._destroy_client = False
         self._remove_account = False
 
-        self._ssl_errors = set()
-
         self._destroyed = False
 
         self.available_transports = {}
@@ -90,35 +103,36 @@ class Client(ConnectionHandlers):
 
         self._create_client()
 
-        self._idle_handler_id = Monitor.connect('state-changed',
-                                                self._idle_state_changed)
-
-        if sys.platform not in ('win32', 'darwin'):
+        if Monitor.is_available():
+            self._idle_handler_id = Monitor.connect('state-changed',
+                                                    self._idle_state_changed)
             self._screensaver_handler_id = app.app.connect(
                 'notify::screensaver-active', self._screensaver_state_changed)
 
-        ConnectionHandlers.__init__(self)
-
-    def _set_state(self, state):
+    def _set_state(self, state: ClientState) -> None:
         log.info('State: %s', state)
         self._state = state
 
     @property
-    def state(self):
+    def state(self) -> ClientState:
         return self._state
 
     @property
-    def status(self):
+    def account(self) -> str:
+        return self._account
+
+    @property
+    def status(self) -> str:
         return self._status
 
     @property
-    def status_message(self):
+    def status_message(self) -> str:
         if self._idle_status_active():
             return self._idle_status_message
         return self._status_message
 
     @property
-    def priority(self):
+    def priority(self) -> int:
         return self._priority
 
     @property
@@ -130,19 +144,19 @@ class Client(ConnectionHandlers):
         return self._client.features
 
     @property
-    def local_address(self):
+    def local_address(self) -> Optional[str]:
         address = self._client.local_address
         if address is not None:
             return address.to_string().split(':')[0]
         return None
 
-    def set_remove_account(self, value):
+    def set_remove_account(self, value: bool) -> None:
         # Used by the RemoveAccount Assistant to make the Client
         # not react to any stream errors that happen while the
         # account is removed by the server and the connection is killed
         self._remove_account = value
 
-    def _create_client(self):
+    def _create_client(self) -> None:
         if self._destroyed:
             # If we disable an account cleanup() is called and all
             # modules are unregistered. Because disable_account() does not wait
@@ -160,36 +174,16 @@ class Client(ConnectionHandlers):
         self._client.set_username(self._user)
         self._client.set_resource(get_resource(self._account))
 
-        custom_host = get_custom_host(self._account)
-        if custom_host is not None:
-            self._client.set_custom_host(*custom_host)
-
-        pass_saved = app.config.get_per('accounts', self._account, 'savepass')
+        pass_saved = app.settings.get_account_setting(self._account, 'savepass')
         if pass_saved:
             # Request password from keyring only if the user chose to save
             # his password
             self.password = passwords.get_password(self._account)
 
-        anonymous = app.config.get_per(
-            'accounts', self._account, 'anonymous_auth')
-        if anonymous:
-            self._client.set_mechs(['ANONYMOUS'])
-
         self._client.set_password(self.password)
         self._client.set_accepted_certificates(
             app.cert_store.get_certificates())
 
-        self._client.set_ignored_tls_errors(
-            get_ignored_ssl_errors(self._account))
-
-        if app.config.get_per('accounts', self._account,
-                              'use_plain_connection'):
-            self._client.set_connection_types([ConnectionType.PLAIN])
-
-        proxy = get_user_proxy(self._account)
-        if proxy is not None:
-            self._client.set_proxy(proxy)
-
         self._client.subscribe('resume-failed', self._on_resume_failed)
         self._client.subscribe('resume-successful', self._on_resume_successful)
         self._client.subscribe('disconnected', self._on_disconnected)
@@ -202,45 +196,35 @@ class Client(ConnectionHandlers):
         for handler in modules.get_handlers(self):
             self._client.register_handler(handler)
 
-    def process_ssl_errors(self):
-        if not self._ssl_errors:
-            self.connect(ignore_all_errors=True)
-            return
+    def _on_resume_failed(self,
+                          _client: NBXMPPClient,
+                          _signal_name: str) -> None:
 
-        open_window('SSLErrorDialog',
-                    account=self._account,
-                    connection=self,
-                    cert=self._client.peer_certificate[0],
-                    error_num=self._ssl_errors.pop())
-
-    def _on_resume_failed(self, _client, _signal_name):
         log.info('Resume failed')
-        app.nec.push_incoming_event(NetworkEvent(
-            'our-show', account=self._account, show='offline'))
-        self.get_module('Chatstate').enabled = False
+        self.notify('resume-failed')
+
+    def _on_resume_successful(self,
+                              _client: NBXMPPClient,
+                              _signal_name: str) -> None:
 
-    def _on_resume_successful(self, _client, _signal_name):
         self._set_state(ClientState.CONNECTED)
         self._set_client_available()
 
         if self._status_sync_on_resume:
             self._status_sync_on_resume = False
             self.update_presence()
-        else:
-            # Normally show is updated when we receive a presence reflection.
-            # On resume, if show has not changed while offline, we dont send
-            # a new presence so we have to trigger the event here.
-            app.nec.push_incoming_event(
-                NetworkEvent('our-show',
-                             account=self._account,
-                             show=self._status))
 
-    def _set_client_available(self):
+        self.notify('state-changed', SimpleClientState.CONNECTED)
+
+    def _set_client_available(self) -> None:
         self._set_state(ClientState.AVAILABLE)
-        app.nec.push_incoming_event(NetworkEvent('account-connected',
-                                                 account=self._account))
+        app.ged.raise_event(AccountConnected(account=self._account))
+
+    def disconnect(self,
+                   gracefully: bool,
+                   reconnect: bool,
+                   destroy_client: bool = False) -> None:
 
-    def disconnect(self, gracefully, reconnect, destroy_client=False):
         if self._state.is_disconnecting:
             log.warning('Disconnect already in progress')
             return
@@ -252,7 +236,10 @@ class Client(ConnectionHandlers):
         log.info('Starting to disconnect %s', self._account)
         self._client.disconnect(immediate=not gracefully)
 
-    def _on_disconnected(self, _client, _signal_name):
+    def _on_disconnected(self,
+                         _client: NBXMPPClient,
+                         _signal_name: str) -> None:
+
         log.info('Disconnect %s', self._account)
         self._set_state(ClientState.DISCONNECTED)
 
@@ -263,58 +250,64 @@ class Client(ConnectionHandlers):
             self._reconnect = False
 
         elif domain == StreamError.BAD_CERTIFICATE:
-            self._ssl_errors = self._client.peer_certificate[1]
-            self.get_module('Chatstate').enabled = False
             self._reconnect = False
-            self._after_disconnect()
-            app.nec.push_incoming_event(NetworkEvent(
-                'our-show', account=self._account, show='offline'))
-            self.process_ssl_errors()
+            self._destroy_client = True
+
+            cert, errors = self._client.peer_certificate
+
+            open_window('SSLErrorDialog',
+                        account=self._account,
+                        client=self,
+                        cert=cert,
+                        error=errors.pop())
 
         elif domain in (StreamError.STREAM, StreamError.BIND):
             if error == 'conflict':
                 # Reset resource
-                app.config.set_per('accounts', self._account,
-                                   'resource', 'gajim.$rand')
+                app.settings.set_account_setting(self._account,
+                                                 'resource',
+                                                 'gajim.$rand')
+            if error == 'system-shutdown':
+                app.ged.raise_event(
+                    Notification(account=self._account,
+                                 type='server-shutdown',
+                                 title=_('Server Shutdown'),
+                                 text=text or _('The server was shut down.')))
 
         elif domain == StreamError.SASL:
             self._reconnect = False
+            self._destroy_client = True
 
             if error in ('not-authorized', 'no-password'):
-                def _on_password(password):
+                def _on_password(password: str) -> None:
                     self.password = password
                     self._client.set_password(password)
-                    self.connect()
+                    self._prepare_for_connect()
 
-                app.nec.push_incoming_event(NetworkEvent(
-                    'password-required', conn=self, on_password=_on_password))
+                app.ged.raise_event(PasswordRequired(client=self,
+                                                     on_password=_on_password))
 
-            app.nec.push_incoming_event(
-                NetworkEvent('simple-notification',
-                             account=self._account,
-                             type_='connection-failed',
+            app.ged.raise_event(
+                Notification(account=self._account,
+                             type='connection-failed',
                              title=_('Authentication failed'),
                              text=text or error))
 
         if self._reconnect:
             self._after_disconnect()
             self._schedule_reconnect()
-            app.nec.push_incoming_event(
-                NetworkEvent('our-show', account=self._account, show='error'))
+            if not self._client.resumeable:
+                self.notify('state-changed', SimpleClientState.DISCONNECTED)
+            self.notify('state-changed', SimpleClientState.RESUME_IN_PROGRESS)
 
         else:
-            self.get_module('Chatstate').enabled = False
-            app.nec.push_incoming_event(NetworkEvent(
-                'our-show', account=self._account, show='offline'))
             self._after_disconnect()
+            self.notify('state-changed', SimpleClientState.DISCONNECTED)
 
-    def _after_disconnect(self):
+    def _after_disconnect(self) -> None:
         self._disable_reconnect_timer()
 
-        self.get_module('VCardAvatars').avatar_advertised = False
-
         app.proxy65_manager.disconnect(self._client)
-        self.terminate_sessions()
         self.get_module('Bytestream').remove_all_transfers()
 
         if self._destroy_client:
@@ -323,43 +316,52 @@ class Client(ConnectionHandlers):
             self._destroy_client = False
             self._create_client()
 
-        app.nec.push_incoming_event(NetworkEvent('account-disconnected',
-                                                 account=self._account))
+        app.ged.raise_event(AccountDisonnected(account=self._account))
 
-    def _on_connection_failed(self, _client, _signal_name):
+    def _on_connection_failed(self,
+                              _client: NBXMPPClient,
+                              _signal_name: str) -> None:
         self._schedule_reconnect()
 
-    def _on_connected(self, _client, _signal_name):
+    def _on_connected(self,
+                      _client: NBXMPPClient,
+                      _signal_name: str) -> None:
+
         self._set_state(ClientState.CONNECTED)
         self.get_module('Discovery').discover_server_info()
         self.get_module('Discovery').discover_account_info()
         self.get_module('Discovery').discover_server_items()
-        self.get_module('Chatstate').enabled = True
-        self.get_module('MAM').reset_state()
 
-    def _on_stanza_sent(self, _client, _signal_name, stanza):
-        app.nec.push_incoming_event(NetworkEvent('stanza-sent',
-                                                 account=self._account,
-                                                 stanza=stanza))
+    def _on_stanza_sent(self,
+                        _client: NBXMPPClient,
+                        _signal_name: str,
+                        stanza: Any) -> None:
 
-    def _on_stanza_received(self, _client, _signal_name, stanza):
-        app.nec.push_incoming_event(NetworkEvent('stanza-received',
-                                                 account=self._account,
-                                                 stanza=stanza))
-    def get_own_jid(self):
-        """
+        app.ged.raise_event(StanzaSent(account=self._account,
+                                       stanza=stanza))
+
+    def _on_stanza_received(self,
+                            _client: NBXMPPClient,
+                            _signal_name: str,
+                            stanza: Any) -> None:
+
+        app.ged.raise_event(StanzaReceived(account=self._account,
+                                           stanza=stanza))
+
+    def get_own_jid(self) -> JID:
+        '''
         Return the last full JID we received on a bind event.
         In case we were never connected it returns the bare JID from config.
-        """
+        '''
         if self._client is not None:
             jid = self._client.get_bound_jid()
             if jid is not None:
                 return jid
 
         # This returns the bare jid
-        return nbxmpp.JID(app.get_jid_from_account(self._account))
+        return nbxmpp.JID.from_string(app.get_jid_from_account(self._account))
 
-    def change_status(self, show, message):
+    def change_status(self, show: str, message: str) -> None:
         if not message:
             message = ''
 
@@ -369,8 +371,17 @@ class Client(ConnectionHandlers):
         if show != 'offline':
             self._status = show
 
+            app.settings.set_account_setting(
+                self._account,
+                'last_status',
+                show)
+            app.settings.set_account_setting(
+                self._account,
+                'last_status_msg',
+                helpers.to_one_line(message))
+
         if self._state.is_disconnecting:
-            log.warning('Can\'t change status while '
+            log.warning("Can\'t change status while "
                         'disconnect is in progress')
             return
 
@@ -378,7 +389,7 @@ class Client(ConnectionHandlers):
             if show == 'offline':
                 return
 
-            self.connect()
+            self._prepare_for_connect()
             return
 
         if self._state.is_connecting:
@@ -393,11 +404,13 @@ class Client(ConnectionHandlers):
                 self._destroy_client = True
                 self._abort_reconnect()
             else:
-                self.connect()
+                self._prepare_for_connect()
             return
 
         # We are connected
         if show == 'offline':
+            self.get_module('UserTune').set_tune(None)
+            self.get_module('UserLocation').set_location(None)
             presence = self.get_module('Presence').get_presence(
                 typ='unavailable',
                 status=message,
@@ -411,7 +424,7 @@ class Client(ConnectionHandlers):
 
         self.update_presence()
 
-    def update_presence(self, include_muc=True):
+    def update_presence(self, include_muc: bool = True) -> None:
         status, message, idle = self.get_presence_state()
         self._priority = app.get_priority(self._account, status)
         self.get_module('Presence').send_presence(
@@ -423,49 +436,48 @@ class Client(ConnectionHandlers):
         if include_muc:
             self.get_module('MUC').update_presence()
 
-    def get_module(self, name):
+    def get_module(self, name: str):
         return modules.get(self._account, name)
 
     @helpers.call_counter
-    def connect_machine(self):
+    def connect_machine(self) -> None:
         log.info('Connect machine state: %s', self._connect_machine_calls)
         if self._connect_machine_calls == 1:
-            self.get_module('MetaContacts').get_metacontacts()
-        elif self._connect_machine_calls == 2:
             self.get_module('Delimiter').get_roster_delimiter()
-        elif self._connect_machine_calls == 3:
+        elif self._connect_machine_calls == 2:
             self.get_module('Roster').request_roster()
-        elif self._connect_machine_calls == 4:
+        elif self._connect_machine_calls == 3:
             self._finish_connect()
 
-    def _finish_connect(self):
+    def _finish_connect(self) -> None:
         self._status_sync_on_resume = False
         self._set_client_available()
 
         # We did not resume the stream, so we are not joined any MUCs
         self.update_presence(include_muc=False)
 
-        if not self.avatar_conversion:
-            # ask our VCard
-            self.get_module('VCardTemp').request_vcard()
-
         self.get_module('Bookmarks').request_bookmarks()
         self.get_module('SoftwareVersion').set_enabled(True)
+        self.get_module('EntityTime').set_enabled(True)
+        self.get_module('LastActivity').set_enabled(True)
         self.get_module('Annotations').request_annotations()
         self.get_module('Blocking').get_blocking_list()
 
-        # Inform GUI we just signed in
-        app.nec.push_incoming_event(NetworkEvent(
-            'signed-in', account=self._account, conn=self))
+        if app.settings.get_account_setting(self._account, 'publish_tune'):
+            self.get_module('UserTune').set_enabled(True)
+
+        self.notify('state-changed', SimpleClientState.CONNECTED)
+
+        app.ged.raise_event(SignedIn(account=self._account, conn=self))
         modules.send_stored_publish(self._account)
 
-    def send_stanza(self, stanza):
-        """
+    def send_stanza(self, stanza: Any) -> None:
+        '''
         Send a stanza untouched
-        """
+        '''
         return self._client.send_stanza(stanza)
 
-    def send_message(self, message):
+    def send_message(self, message: OutgoingMessage) -> None:
         if not self._state.is_available:
             log.warning('Trying to send message while offline')
             return
@@ -473,80 +485,99 @@ class Client(ConnectionHandlers):
         stanza = self.get_module('Message').build_message_stanza(message)
         message.stanza = stanza
 
-        method = get_encryption_method(message.account, message.jid)
-        if method is not None:
-            # TODO: Make extension point return encrypted message
-
-            extension = 'encrypt'
-            if message.is_groupchat:
-                extension = 'gc_encrypt'
-            app.plugin_manager.extension_point(extension + method,
-                                               self,
-                                               message,
-                                               self._send_message)
+        method = message.contact.settings.get('encryption')
+        if not method:
+            self._send_message(message)
             return
 
-        self._send_message(message)
+        # TODO: Make extension point return encrypted message
+        extension = 'encrypt'
+        if message.is_groupchat:
+            extension = 'gc_encrypt'
+        app.plugin_manager.extension_point(extension + method,
+                                           self,
+                                           message,
+                                           self._send_message)
 
-    def _send_message(self, message):
+    def _send_message(self, message: OutgoingMessage) -> None:
         message.set_sent_timestamp()
         message.message_id = self.send_stanza(message.stanza)
 
-        app.nec.push_incoming_event(
-            MessageSentEvent(None, jid=message.jid, **vars(message)))
+        app.ged.raise_event(
+            MessageSent(jid=message.jid,
+                        account=message.account,
+                        message=message.message,
+                        chatstate=message.chatstate,
+                        timestamp=message.timestamp,
+                        additional_data=message.additional_data,
+                        label=message.label,
+                        correct_id=message.correct_id,
+                        message_id=message.message_id,
+                        play_sound=message.play_sound))
 
         if message.is_groupchat:
             return
 
         self.get_module('Message').log_message(message)
 
-    def send_messages(self, jids, message):
-        if not self._state.is_available:
-            log.warning('Trying to send message while offline')
-            return
+    def _prepare_for_connect(self) -> None:
+        custom_host = get_custom_host(self._account)
+        if custom_host is not None:
+            self._client.set_custom_host(*custom_host)
 
-        for jid in jids:
-            message = message.copy()
-            message.contact = app.contacts.create_contact(jid, message.account)
-            stanza = self.get_module('Message').build_message_stanza(message)
-            message.stanza = stanza
-            self._send_message(message)
+        gssapi = app.settings.get_account_setting(self._account,
+                                                  'enable_gssapi')
+        if gssapi:
+            self._client.set_mechs(['GSSAPI'])
 
-    def connect(self, ignore_all_errors=False):
+        anonymous = app.settings.get_account_setting(self._account,
+                                                     'anonymous_auth')
+        if anonymous:
+            self._client.set_mechs(['ANONYMOUS'])
+
+        if app.settings.get_account_setting(self._account,
+                                            'use_plain_connection'):
+            self._client.set_connection_types([ConnectionType.PLAIN])
+
+        proxy = get_account_proxy(self._account)
+        if proxy is not None:
+            self._client.set_proxy(proxy)
+
+        self.connect()
+
+    def connect(self, ignored_tls_errors: IgnoredTlsErrorsT = None) -> None:
         if self._state not in (ClientState.DISCONNECTED,
                                ClientState.RECONNECT_SCHEDULED):
             # Do not try to reco while we are already trying
             return
 
         log.info('Connect')
+
+        self._client.set_ignored_tls_errors(ignored_tls_errors)
         self._reconnect = True
         self._disable_reconnect_timer()
         self._set_state(ClientState.CONNECTING)
-        if ignore_all_errors:
-            self._client.set_ignore_tls_errors(True)
-            self._client.connect()
-        else:
-            if warn_about_plain_connection(self._account,
-                                           self._client.connection_types):
-                app.nec.push_incoming_event(NetworkEvent(
-                    'plain-connection',
-                    account=self._account,
-                    connect=self._client.connect,
-                    abort=self._abort_reconnect))
-                return
-            self._client.connect()
+        self.notify('state-changed', SimpleClientState.CONNECTING)
 
-    def _schedule_reconnect(self):
+        if warn_about_plain_connection(self._account,
+                                       self._client.connection_types):
+            app.ged.raise_event(PlainConnection(account=self._account,
+                                                connect=self._client.connect,
+                                                abort=self._abort_reconnect))
+            return
+
+        self._client.connect()
+
+    def _schedule_reconnect(self) -> None:
         self._set_state(ClientState.RECONNECT_SCHEDULED)
-        log.info("Reconnect to %s in 3s", self._account)
+        log.info('Reconnect to %s in 3s', self._account)
         self._reconnect_timer_source = GLib.timeout_add_seconds(
-            3, self.connect)
+            3, self._prepare_for_connect)
 
-    def _abort_reconnect(self):
+    def _abort_reconnect(self) -> None:
         self._set_state(ClientState.DISCONNECTED)
         self._disable_reconnect_timer()
-        app.nec.push_incoming_event(
-            NetworkEvent('our-show', account=self._account, show='offline'))
+        self.notify('state-changed', SimpleClientState.DISCONNECTED)
 
         if self._destroy_client:
             self._client.destroy()
@@ -554,12 +585,12 @@ class Client(ConnectionHandlers):
             self._destroy_client = False
             self._create_client()
 
-    def _disable_reconnect_timer(self):
+    def _disable_reconnect_timer(self) -> None:
         if self._reconnect_timer_source is not None:
             GLib.source_remove(self._reconnect_timer_source)
             self._reconnect_timer_source = None
 
-    def _idle_state_changed(self, monitor):
+    def _idle_state_changed(self, monitor: IdleMonitorManager) -> None:
         state = monitor.state.value
 
         if monitor.is_awake():
@@ -568,7 +599,7 @@ class Client(ConnectionHandlers):
             self._update_status()
             return
 
-        if not app.config.get(f'auto{state}'):
+        if not app.settings.get(f'auto{state}'):
             return
 
         if (state in ('away', 'xa') and self._status == 'online' or
@@ -579,7 +610,7 @@ class Client(ConnectionHandlers):
                 state, self._status_message)
             self._update_status()
 
-    def _update_status(self):
+    def _update_status(self) -> None:
         if not self._idle_status_enabled:
             return
 
@@ -589,7 +620,7 @@ class Client(ConnectionHandlers):
         else:
             self._status_sync_on_resume = True
 
-    def _idle_status_active(self):
+    def _idle_status_active(self) -> bool:
         if not Monitor.is_available():
             return False
 
@@ -598,20 +629,23 @@ class Client(ConnectionHandlers):
 
         return self._idle_status != 'online'
 
-    def get_presence_state(self):
+    def get_presence_state(self) -> tuple[str, str, bool]:
         if self._idle_status_active():
             return self._idle_status, self._idle_status_message, True
         return self._status, self._status_message, False
 
     @staticmethod
-    def _screensaver_state_changed(application, _param):
+    def _screensaver_state_changed(application: Gtk.Application,
+                                   _param: GObject.ParamSpec) -> None:
         active = application.get_property('screensaver-active')
         Monitor.set_extended_away(active)
 
-    def cleanup(self):
+    def cleanup(self) -> None:
+        self.disconnect_signals()
         self._destroyed = True
-        Monitor.disconnect(self._idle_handler_id)
-        app.app.disconnect(self._screensaver_handler_id)
+        if Monitor.is_available():
+            Monitor.disconnect(self._idle_handler_id)
+            app.app.disconnect(self._screensaver_handler_id)
         if self._client is not None:
             # cleanup() is called before nbmxpp.Client has disconnected,
             # when we disable the account. So we need to unregister
@@ -621,7 +655,7 @@ class Client(ConnectionHandlers):
                 self._client.unregister_handler(handler)
         modules.unregister_modules(self)
 
-    def quit(self, kill_core):
+    def quit(self, kill_core: bool) -> None:
         if kill_core and self._state in (ClientState.CONNECTING,
                                          ClientState.CONNECTED,
                                          ClientState.AVAILABLE):
