@@ -14,25 +14,44 @@
 
 # Presence handler
 
+from __future__ import annotations
+
+from typing import Any
+from typing import Optional
+from typing import Union
+
 import time
 
 import nbxmpp
-from nbxmpp.namespaces import Namespace
-from nbxmpp.structs import StanzaHandler
 from nbxmpp.const import PresenceType
+from nbxmpp.namespaces import Namespace
+from nbxmpp.protocol import JID
+from nbxmpp.structs import PresenceProperties
+from nbxmpp.structs import StanzaHandler
 
 from gajim.common import app
 from gajim.common import idle
+from gajim.common import types
+from gajim.common.events import PresenceReceived
+from gajim.common.events import ShowChanged
+from gajim.common.events import SubscribePresenceReceived
+from gajim.common.events import SubscribedPresenceReceived
+from gajim.common.events import UnsubscribedPresenceReceived
 from gajim.common.i18n import _
-from gajim.common.nec import NetworkEvent
-from gajim.common.helpers import should_log
+from gajim.common.structs import PresenceData
 from gajim.common.const import KindConstant
 from gajim.common.const import ShowConstant
 from gajim.common.modules.base import BaseModule
 
 
 class Presence(BaseModule):
-    def __init__(self, con):
+
+    _nbxmpp_extends = 'BasePresence'
+    _nbxmpp_methods = [
+        'subscribed',
+    ]
+
+    def __init__(self, con: types.Client) -> None:
         BaseModule.__init__(self, con)
 
         self.handlers = [
@@ -57,19 +76,25 @@ class Presence(BaseModule):
                           priority=49),
         ]
 
+        self._presence_store: dict[JID, PresenceData] = {}
+
         # keep the jids we auto added (transports contacts) to not send the
         # SUBSCRIBED event to GUI
-        self.automatically_added = []
+        self.automatically_added: list[str] = []
 
         # list of jid to auto-authorize
-        self._jids_for_auto_auth = set()
+        self._jids_for_auto_auth: set[str] = set()
 
-    def _presence_received(self, _con, stanza, properties):
+    def _presence_received(self,
+                           _con: types.xmppClient,
+                           stanza: nbxmpp.protocol.Presence,
+                           properties: PresenceProperties
+                           ) -> None:
         if properties.from_muc:
             # MUC occupant presences are already handled in MUC module
             return
 
-        muc = self._con.get_module('MUC').get_manager().get(properties.jid)
+        muc = self._con.get_module('MUC').get_muc_data(properties.jid)
         if muc is not None:
             # Presence from the MUC itself, used for MUC avatar
             # handled in VCardAvatars module
@@ -77,22 +102,19 @@ class Presence(BaseModule):
 
         self._log.info('Received from %s', properties.jid)
 
+        presence_data = PresenceData.from_presence(properties)
+        self._presence_store[properties.jid] = presence_data
+
+        contact = self._con.get_module('Contacts').get_contact(properties.jid)
+        contact.update_presence(presence_data)
+
         if properties.type == PresenceType.ERROR:
             self._log.info('Error: %s %s', properties.jid, properties.error)
             return
 
-        if self._account == 'Local':
-            app.nec.push_incoming_event(
-                NetworkEvent('raw-pres-received',
-                             conn=self._con,
-                             stanza=stanza))
-            return
-
         if properties.is_self_presence:
-            app.nec.push_incoming_event(
-                NetworkEvent('our-show',
-                             account=self._account,
-                             show=properties.show.value))
+            app.ged.raise_event(ShowChanged(account=self._account,
+                                            show=properties.show.value))
             return
 
         jid = properties.jid.bare
@@ -107,14 +129,15 @@ class Presence(BaseModule):
         if properties.type.is_unavailable:
             show = 'offline'
 
-        event_attrs = {
+        event_attrs: dict[str, Any] = {
+            'account': self._account,
             'conn': self._con,
             'stanza': stanza,
             'prio': properties.priority,
             'need_add_in_roster': False,
             'popup': False,
             'ptype': properties.type.value,
-            'jid': properties.jid.bare,
+            'jid': properties.jid.new_as_bare(),
             'resource': properties.jid.resource,
             'id_': properties.id,
             'fjid': str(properties.jid),
@@ -130,107 +153,13 @@ class Presence(BaseModule):
             'contact': None,
         }
 
-        event_ = NetworkEvent('presence-received', **event_attrs)
+        app.ged.raise_event(PresenceReceived(**event_attrs))
 
-        # TODO: Refactor
-        self._update_contact(event_, properties)
-
-        app.nec.push_incoming_event(event_)
-
-    def _update_contact(self, event, properties):
-        # Note: A similar method also exists in connection_zeroconf
-        jid = properties.jid.bare
-        resource = properties.jid.resource
-
-        status_strings = ['offline', 'error', 'online', 'chat', 'away',
-                          'xa', 'dnd']
-
-        event.new_show = status_strings.index(event.show)
-
-        # Update contact
-        contact_list = app.contacts.get_contacts(self._account, jid)
-        if not contact_list:
-            self._log.warning('No contact found')
-            return
-
-        event.contact_list = contact_list
-
-        contact = app.contacts.get_contact_strict(self._account,
-                                                  properties.jid.bare,
-                                                  properties.jid.resource)
-        if contact is None:
-            contact = app.contacts.get_first_contact_from_jid(self._account,
-                                                              jid)
-            if contact is None:
-                self._log.warning('First contact not found')
-                return
-
-            if (self._is_resource_known(contact_list) and
-                    not app.jid_is_transport(jid)):
-                # Another resource of an existing contact connected
-                # Add new contact
-                event.old_show = 0
-                contact = app.contacts.copy_contact(contact)
-                contact.resource = resource
-                app.contacts.add_contact(self._account, contact)
-            else:
-                # Convert the initial roster contact to a contact with resource
-                contact.resource = resource
-                event.old_show = 0
-                if contact.show in status_strings:
-                    event.old_show = status_strings.index(contact.show)
-
-            event.need_add_in_roster = True
-
-        elif contact.show in status_strings:
-            event.old_show = status_strings.index(contact.show)
-
-        # Update contact with presence data
-        contact.show = event.show
-        contact.status = properties.status
-        contact.priority = properties.priority
-        contact.idle_time = properties.idle_timestamp
-
-        event.contact = contact
-
-        if not app.jid_is_transport(jid) and len(contact_list) == 1:
-            # It's not an agent
-            if event.old_show == 0 and event.new_show > 1:
-                if not jid in app.newly_added[self._account]:
-                    app.newly_added[self._account].append(jid)
-                if jid in app.to_be_removed[self._account]:
-                    app.to_be_removed[self._account].remove(jid)
-            elif event.old_show > 1 and event.new_show == 0 and \
-            self._con.state.is_available:
-                if not jid in app.to_be_removed[self._account]:
-                    app.to_be_removed[self._account].append(jid)
-                if jid in app.newly_added[self._account]:
-                    app.newly_added[self._account].remove(jid)
-
-        if app.jid_is_transport(jid):
-            return
-
-        if properties.type.is_unavailable:
-            # TODO: This causes problems when another
-            # resource signs off!
-            self._con.get_module('Bytestream').stop_all_active_file_transfers(
-                contact)
         self._log_presence(properties)
 
-    @staticmethod
-    def _is_resource_known(contact_list):
-        if len(contact_list) > 1:
-            return True
-
-        if contact_list[0].resource == '':
-            return False
-        return contact_list[0].show not in ('not in roster', 'offline')
-
-    def _log_presence(self, properties):
+    def _log_presence(self, properties: PresenceProperties) -> None:
         if not app.settings.get('log_contact_status_changes'):
             return
-        if not should_log(self._account, properties.jid.bare):
-            return
 
         show = ShowConstant[properties.show.name]
         if properties.type.is_unavailable:
@@ -243,7 +172,11 @@ class Presence(BaseModule):
                                              message=properties.status,
                                              show=show)
 
-    def _subscribe_received(self, _con, _stanza, properties):
+    def _subscribe_received(self,
+                            _con: types.xmppClient,
+                            _stanza: nbxmpp.protocol.Presence,
+                            properties: PresenceProperties
+                            ) -> None:
         jid = properties.jid.bare
         fjid = str(properties.jid)
 
@@ -256,17 +189,16 @@ class Presence(BaseModule):
                        auto_auth, properties.nickname)
 
         if auto_auth or jid in self._jids_for_auto_auth:
-            self.send_presence(fjid, 'subscribed')
-            self._jids_for_auto_auth.discard(jid)
             self._log.info('Auto respond with subscribed: %s', jid)
+            self.subscribed(jid)
             return
 
         status = (properties.status or
                   _('I would like to add you to my roster.'))
 
-        app.nec.push_incoming_event(NetworkEvent(
-            'subscribe-presence-received',
+        app.ged.raise_event(SubscribePresenceReceived(
             conn=self._con,
+            account=self._account,
             jid=jid,
             fjid=fjid,
             status=status,
@@ -275,84 +207,79 @@ class Presence(BaseModule):
 
         raise nbxmpp.NodeProcessed
 
-    def _subscribed_received(self, _con, _stanza, properties):
+    def _subscribed_received(self,
+                             _con: types.xmppClient,
+                             _stanza: nbxmpp.protocol.Presence,
+                             properties: PresenceProperties
+                             ) -> None:
         jid = properties.jid.bare
         self._log.info('Received Subscribed: %s', properties.jid)
         if jid in self.automatically_added:
             self.automatically_added.remove(jid)
             raise nbxmpp.NodeProcessed
 
-        app.nec.push_incoming_event(NetworkEvent(
-            'subscribed-presence-received',
+        app.ged.raise_event(SubscribedPresenceReceived(
             account=self._account,
             jid=properties.jid))
         raise nbxmpp.NodeProcessed
 
-    def _unsubscribe_received(self, _con, _stanza, properties):
+    def _unsubscribe_received(self,
+                              _con: types.xmppClient,
+                              _stanza: nbxmpp.protocol.Presence,
+                              properties: PresenceProperties
+                              ) -> None:
         self._log.info('Received Unsubscribe: %s', properties.jid)
         raise nbxmpp.NodeProcessed
 
-    def _unsubscribed_received(self, _con, _stanza, properties):
+    def _unsubscribed_received(self,
+                               _con: types.xmppClient,
+                               _stanza: nbxmpp.protocol.Presence,
+                               properties: PresenceProperties
+                               ) -> None:
         self._log.info('Received Unsubscribed: %s', properties.jid)
-        app.nec.push_incoming_event(NetworkEvent(
-            'unsubscribed-presence-received',
-            conn=self._con, jid=properties.jid.bare))
+        app.ged.raise_event(UnsubscribedPresenceReceived(
+            conn=self._con,
+            account=self._account,
+            jid=properties.jid.bare))
         raise nbxmpp.NodeProcessed
 
-    def subscribed(self, jid):
-        if not app.account_is_available(self._account):
-            return
-        self._log.info('Subscribed: %s', jid)
-        self.send_presence(jid, 'subscribed')
-
-    def unsubscribed(self, jid):
-        if not app.account_is_available(self._account):
-            return
-
+    def unsubscribed(self, jid: Union[JID, str]) -> None:
         self._log.info('Unsubscribed: %s', jid)
-        self._jids_for_auto_auth.discard(jid)
-        self.send_presence(jid, 'unsubscribed')
+        self._jids_for_auto_auth.discard(str(jid))
+        self._nbxmpp('BasePresence').unsubscribed(jid)
 
-    def unsubscribe(self, jid, remove_auth=True):
-        if not app.account_is_available(self._account):
-            return
-        if remove_auth:
-            self._con.get_module('Roster').del_item(jid)
-        else:
-            self._log.info('Unsubscribe from %s', jid)
-            self._jids_for_auto_auth.discard(jid)
-            self._con.get_module('Roster').unsubscribe(jid)
-            self._con.get_module('Roster').set_item(jid)
-
-    def subscribe(self, jid, msg=None, name='', groups=None, auto_auth=False):
-        if not app.account_is_available(self._account):
-            return
-        if groups is None:
-            groups = []
+    def unsubscribe(self, jid: Union[JID, str]) -> None:
+        self._log.info('Unsubscribe from %s', jid)
+        self._jids_for_auto_auth.discard(str(jid))
+        self._nbxmpp('BasePresence').unsubscribe(jid)
 
+    def subscribe(self,
+                  jid: Union[JID, str],
+                  msg: Optional[str] = None,
+                  name: Optional[str] = None,
+                  groups: Optional[list[str]] = None,
+                  auto_auth: bool = False
+                  ) -> None:
         self._log.info('Request Subscription to %s', jid)
 
         if auto_auth:
             self._jids_for_auto_auth.add(jid)
 
-        infos = {'jid': jid}
-        if name:
-            infos['name'] = name
-        iq = nbxmpp.Iq('set', Namespace.ROSTER)
-        query = iq.setQuery()
-        item = query.addChild('item', attrs=infos)
-        for group in groups:
-            item.addChild('group').setData(group)
-        self._con.connection.send(iq)
+        self._con.get_module('Roster').set_item(jid, name, groups=groups)
+        self._nbxmpp('BasePresence').subscribe(jid,
+                                               status=msg,
+                                               nick=app.nicks[self._account])
 
-        self.send_presence(jid,
-                           'subscribe',
-                           status=msg,
-                           nick=app.nicks[self._account])
-
-    def get_presence(self, to=None, typ=None, priority=None,
-                     show=None, status=None, nick=None, caps=True,
-                     idle_time=False):
+    def get_presence(self,
+                     to: Optional[str] = None,
+                     typ: Optional[str] = None,
+                     priority: Optional[int] = None,
+                     show: Optional[str] = None,
+                     status: Optional[str] = None,
+                     nick: Optional[str] = None,
+                     caps: bool = True,
+                     idle_time: bool = False
+                     ) -> nbxmpp.Presence:
         if show not in ('chat', 'away', 'xa', 'dnd'):
             # Gajim sometimes passes invalid show values here
             # until this is fixed this is a workaround
@@ -380,7 +307,7 @@ class Presence(BaseModule):
 
         return presence
 
-    def send_presence(self, *args, **kwargs):
+    def send_presence(self, *args: Any, **kwargs: Any) -> None:
         if not app.account_is_connected(self._account):
             return
         presence = self.get_presence(*args, **kwargs)
@@ -388,7 +315,3 @@ class Presence(BaseModule):
             'send-presence', self._account, presence)
         self._log.debug('Send presence:\n%s', presence)
         self._con.connection.send(presence)
-
-
-def get_instance(*args, **kwargs):
-    return Presence(*args, **kwargs), 'Presence'
