@@ -14,28 +14,48 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
+from typing import Any
+from typing import Callable
+from typing import Optional
+from typing import Union
+from typing import cast
+
+import logging
+
 from gi.repository import Gtk
 from gi.repository import GLib
 from gi.repository import Gdk
 from gi.repository import Pango
 
+from nbxmpp.protocol import JID
+
 from gajim.common import app
 from gajim.common import passwords
 from gajim.common.i18n import _
 from gajim.common.i18n import Q_
-
-from gajim import gtkgui_helpers
+from gajim.common.setting_values import AllSettingsT
 
 from .util import get_image_button
 from .util import MaxWidthComboBoxText
 from .util import open_window
+from .const import Setting
 from .const import SettingKind
 from .const import SettingType
 
+log = logging.getLogger('gajim.gui.settings')
+
 
 class SettingsDialog(Gtk.ApplicationWindow):
-    def __init__(self, parent, title, flags, settings, account,
-                 extend=None):
+    def __init__(self,
+                 parent: Gtk.Window,
+                 title: str,
+                 flags: Gtk.DialogFlags,
+                 settings: list[Setting],
+                 account: str,
+                 extend: Optional[dict[SettingKind, GenericSetting]] = None
+                 ) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_application(app.app)
         self.set_show_menubar(False)
@@ -63,24 +83,32 @@ class SettingsDialog(Gtk.ApplicationWindow):
 
         self.show_all()
         self.connect_after('key-press-event', self.on_key_press)
+        self.connect_after('destroy', self.__on_destroy)
 
-    def on_key_press(self, _widget, event):
+    def __on_destroy(self, widget: SettingsDialog) -> None:
+        app.check_finalize(self)
+
+    def on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
             self.destroy()
 
-    def get_setting(self, name):
+    def get_setting(self, name: str):
         return self.listbox.get_setting(name)
 
 
 class SettingsBox(Gtk.ListBox):
-    def __init__(self, account=None, jid=None, extend=None):
+    def __init__(self,
+                 account: Optional[str] = None,
+                 jid: Optional[str] = None,
+                 extend: Optional[dict[SettingKind, GenericSetting]] = None
+                 ) -> None:
         Gtk.ListBox.__init__(self)
         self.get_style_context().add_class('settings-box')
         self.account = account
         self.jid = jid
-        self.named_settings = {}
+        self.named_settings: dict[str, GenericSetting] = {}
 
-        self.map = {
+        self.map: dict[SettingKind, GenericSetting] = {
             SettingKind.SWITCH: SwitchSetting,
             SettingKind.SPIN: SpinSetting,
             SettingKind.DIALOG: DialogSetting,
@@ -106,12 +134,19 @@ class SettingsBox(Gtk.ListBox):
                 self.map[setting] = callback
 
         self.connect('row-activated', self.on_row_activated)
+        self.connect('destroy', self.__on_destroy)
 
     @staticmethod
-    def on_row_activated(_listbox, row):
+    def __on_destroy(widget: SettingsBox) -> None:
+        app.check_finalize(widget)
+        for row in widget.get_children():
+            app.check_finalize(row)
+
+    @staticmethod
+    def on_row_activated(_listbox: SettingsBox, row: GenericSetting) -> None:
         row.on_row_activated()
 
-    def add_setting(self, setting):
+    def add_setting(self, setting: Setting) -> None:
         if not isinstance(setting, Gtk.ListBoxRow):
             if setting.props is not None:
                 listitem = self.map[setting.kind](self.account,
@@ -127,28 +162,29 @@ class SettingsBox(Gtk.ListBox):
             self.named_settings[setting.name] = listitem
         self.add(listitem)
 
-    def get_setting(self, name):
+    def get_setting(self, name: str) -> GenericSetting:
         return self.named_settings[name]
 
-    def update_states(self):
-        for row in self.get_children():
+    def update_states(self) -> None:
+        for row in cast(list[GenericSetting], self.get_children()):
             row.update_activatable()
 
 
 class GenericSetting(Gtk.ListBoxRow):
     def __init__(self,
-                 account,
-                 jid,
-                 label,
-                 type_,
-                 value,
-                 name,
-                 callback,
-                 data,
-                 desc,
-                 bind,
-                 inverted,
-                 enabled_func):
+                 account: str,
+                 jid: JID,
+                 label: str,
+                 type_: SettingType,
+                 value: AllSettingsT,
+                 name: Optional[str],
+                 callback: Optional[Callable[..., None]] = None,
+                 data: Optional[Any] = None,
+                 desc: Optional[str] = None,
+                 bind: Optional[str] = None,
+                 inverted: bool = False,
+                 enabled_func: Optional[Callable[..., bool]] = None
+                 ) -> None:
 
         Gtk.ListBoxRow.__init__(self)
         self._grid = Gtk.Grid()
@@ -168,6 +204,16 @@ class GenericSetting(Gtk.ListBoxRow):
         self.enabled_func = enabled_func
         self.setting_value = self.get_value()
 
+        self._locked_icon = Gtk.Image.new_from_icon_name(
+            'feather-lock-symbolic',
+            Gtk.IconSize.MENU)
+        self._locked_icon.set_visible(False)
+        self._locked_icon.set_no_show_all(True)
+        self._locked_icon.set_halign(Gtk.Align.END)
+        self._locked_icon.set_tooltip_text(_('Setting is locked by the system'))
+
+        self._grid.add(self._locked_icon)
+
         description_box = Gtk.Box(
             orientation=Gtk.Orientation.VERTICAL, spacing=0)
         description_box.set_valign(Gtk.Align.CENTER)
@@ -202,7 +248,7 @@ class GenericSetting(Gtk.ListBoxRow):
 
         self._bind_sensitive_state()
 
-    def _bind_sensitive_state(self):
+    def _bind_sensitive_state(self) -> None:
         if self.bind is None:
             return
 
@@ -231,7 +277,11 @@ class GenericSetting(Gtk.ListBoxRow):
             value = not value
         self.set_sensitive(value)
 
-    def _parse_bind(self):
+    def _parse_bind(self) -> tuple[SettingType,
+                                   str,
+                                   Optional[str],
+                                   Optional[JID]]:
+        assert self.bind is not None
         if '::' not in self.bind:
             return SettingType.CONFIG, self.bind, None, None
 
@@ -253,7 +303,11 @@ class GenericSetting(Gtk.ListBoxRow):
                                 self.jid)
 
     @staticmethod
-    def __get_value(type_, value, account, jid):
+    def __get_value(type_: SettingType,
+                    value: AllSettingsT,
+                    account: str,
+                    jid: JID
+                    ) -> Optional[AllSettingsT]:
         if value is None:
             return None
         if type_ == SettingType.VALUE:
@@ -272,28 +326,24 @@ class GenericSetting(Gtk.ListBoxRow):
         if type_ == SettingType.ACCOUNT_CONFIG:
             if value == 'password':
                 return passwords.get_password(account)
-            if value == 'no_log_for':
-                no_log = app.settings.get_account_setting(
-                    account, 'no_log_for').split()
-                return account not in no_log
             return app.settings.get_account_setting(account, value)
 
         if type_ == SettingType.ACTION:
+            assert isinstance(value, str)
             if value.startswith('-'):
                 return account + value
             return value
 
         raise ValueError('Wrong SettingType?')
 
-    def set_value(self, state):
+    def set_value(self, state: AllSettingsT) -> None:
         if self.type_ == SettingType.CONFIG:
             app.settings.set(self.value, state)
 
         elif self.type_ == SettingType.ACCOUNT_CONFIG:
             if self.value == 'password':
+                assert isinstance(state, str)
                 passwords.save_password(self.account, state)
-            if self.value == 'no_log_for':
-                self.set_no_log_for(self.account, state)
             else:
                 app.settings.set_account_setting(self.account,
                                                  self.value,
@@ -310,21 +360,17 @@ class GenericSetting(Gtk.ListBoxRow):
         if self.callback is not None:
             self.callback(state, self.data)
 
-    @staticmethod
-    def set_no_log_for(account, state):
-        no_log = app.settings.get_account_setting(account, 'no_log_for').split()
-        if state and account in no_log:
-            no_log.remove(account)
-        elif not state and account not in no_log:
-            no_log.append(account)
-        app.settings.set_account_setting(account,
-                                         'no_log_for',
-                                         ' '.join(no_log))
-
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         raise NotImplementedError
 
-    def update_activatable(self):
+    def update_activatable(self) -> None:
+        if self.type_ == SettingType.CONFIG:
+            if app.settings.has_app_override(self.value):
+                self.set_activatable(False)
+                self.set_sensitive(False)
+                self._locked_icon.show()
+                return
+
         if self.enabled_func is None:
             return
 
@@ -332,11 +378,15 @@ class GenericSetting(Gtk.ListBoxRow):
         self.set_activatable(enabled_func_value)
         self.set_sensitive(enabled_func_value)
 
-    def _add_action_button(self, kwargs):
-        icon_name = kwargs.get('button-icon-name')
-        button_text = kwargs.get('button-text')
-        tooltip_text = kwargs.get('button-tooltip') or ''
-        style = kwargs.get('button-style')
+    def _add_action_button(self,
+                           kwargs: dict[str, Union[str,
+                                                   Callable[..., None],
+                                                   None]]
+                           ) -> None:
+        icon_name = cast(str, kwargs.get('button-icon-name'))
+        button_text = cast(str, kwargs.get('button-text'))
+        tooltip_text = cast(str, kwargs.get('button-tooltip') or '')
+        style = cast(str, kwargs.get('button-style'))
 
         if icon_name is not None:
             button = Gtk.Button.new_from_icon_name(icon_name, Gtk.IconSize.MENU)
@@ -351,18 +401,21 @@ class GenericSetting(Gtk.ListBoxRow):
             for css_class in style.split(' '):
                 button.get_style_context().add_class(css_class)
 
-        button.connect('clicked', kwargs['button-callback'])
+        callback = kwargs['button-callback']
+        assert isinstance(callback, Callable)
+        button.connect('clicked', callback)
         button.set_tooltip_text(tooltip_text)
         self.setting_box.add(button)
 
 
 class SwitchSetting(GenericSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         GenericSetting.__init__(self, *args)
 
         self.switch = Gtk.Switch()
         if self.type_ == SettingType.ACTION:
-            self.switch.set_action_name('app.%s' % self.setting_value)
+            self.switch.set_action_name(f'app.{self.setting_value}')
+            assert isinstance(self.setting_value, str)
             state = app.app.get_action_state(self.setting_value)
             self.switch.set_active(state.get_boolean())
         else:
@@ -387,22 +440,22 @@ class SwitchSetting(GenericSetting):
 
         self.show_all()
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         state = self.switch.get_active()
         self.switch.set_active(not state)
 
-    def on_switch(self, switch, *args):
+    def on_switch(self, switch: Gtk.Switch, *args: Any) -> None:
         value = switch.get_active()
         self.set_value(value)
         self._set_label(value)
 
-    def _set_label(self, active):
+    def _set_label(self, active: bool) -> None:
         text = Q_('?switch:On') if active else Q_('?switch:Off')
         self._switch_state_label.set_text(text)
 
 
 class EntrySetting(GenericSetting):
-    def __init__(self, *args):
+    def __init__(self, *args: Any) -> None:
         GenericSetting.__init__(self, *args)
 
         self.entry = Gtk.Entry()
@@ -417,21 +470,36 @@ class EntrySetting(GenericSetting):
 
         self.setting_box.pack_end(self.entry, True, True, 0)
 
+        assert isinstance(self.value, str)
+        app.settings.connect_signal(self.value,
+                                    self._on_setting_changed,
+                                    account=self.account,
+                                    jid=self.jid)
+
+        self.connect('destroy', self._on_destroy)
+
         self.show_all()
 
-    def on_text_change(self, *args):
+    def _on_setting_changed(self, value: str, *args: Any) -> None:
+        self.entry.set_text(value)
+
+    def _on_destroy(self, *args: Any) -> None:
+        app.settings.disconnect_signals(self)
+
+    def on_text_change(self, *args: Any) -> None:
         text = self.entry.get_text()
         self.set_value(text)
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         self.entry.grab_focus()
 
 
 class ColorSetting(GenericSetting):
-    def __init__(self, *args):
+    def __init__(self, *args: Any) -> None:
         GenericSetting.__init__(self, *args)
 
         rgba = Gdk.RGBA()
+        assert isinstance(self.setting_value, str)
         rgba.parse(self.setting_value)
         self.color_button = Gtk.ColorButton()
         self.color_button.set_rgba(rgba)
@@ -441,19 +509,35 @@ class ColorSetting(GenericSetting):
 
         self.setting_box.pack_end(self.color_button, True, True, 0)
 
+        assert isinstance(self.value, str)
+        app.settings.connect_signal(self.value,
+                                    self._on_setting_changed,
+                                    account=self.account,
+                                    jid=self.jid)
+
+        self.connect('destroy', self._on_destroy)
+
         self.show_all()
 
-    def on_color_set(self, button):
+    def _on_setting_changed(self, value: str, *args: Any) -> None:
+        rgba = Gdk.RGBA()
+        rgba.parse(value)
+        self.color_button.set_rgba(rgba)
+
+    def _on_destroy(self, *args: Any) -> None:
+        app.settings.disconnect_signals(self)
+
+    def on_color_set(self, button: Gtk.ColorButton) -> None:
         rgba = button.get_rgba()
         self.set_value(rgba.to_string())
         app.css_config.refresh()
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         self.color_button.grab_focus()
 
 
 class DialogSetting(GenericSetting):
-    def __init__(self, *args, dialog):
+    def __init__(self, *args: Any, dialog: SettingsDialog) -> None:
         GenericSetting.__init__(self, *args)
         self.dialog = dialog
 
@@ -464,24 +548,26 @@ class DialogSetting(GenericSetting):
 
         self.show_all()
 
-    def show_dialog(self, parent):
+    def show_dialog(self, parent: Gtk.Window) -> None:
         if self.dialog:
             dialog = self.dialog(self.account, parent)
             dialog.connect('destroy', self.on_destroy)
 
-    def on_destroy(self, *args):
+    def on_destroy(self, *args: Any) -> None:
         self.setting_value.set_text(self.get_setting_value())
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         self.setting_value.hide()
         return ''
 
-    def on_row_activated(self):
-        self.show_dialog(self.get_toplevel())
+    def on_row_activated(self) -> None:
+        window = self.get_toplevel()
+        assert isinstance(window, Gtk.Window)
+        self.show_dialog(window)
 
 
 class SpinSetting(GenericSetting):
-    def __init__(self, *args, range_):
+    def __init__(self, *args: Any, range_: tuple[float, float]) -> None:
         GenericSetting.__init__(self, *args)
 
         lower, upper = range_
@@ -496,7 +582,9 @@ class SpinSetting(GenericSetting):
         self.spin.set_adjustment(adjustment)
         self.spin.set_numeric(True)
         self.spin.set_update_policy(Gtk.SpinButtonUpdatePolicy.IF_VALID)
-        self.spin.set_value(self.setting_value)
+
+        assert self.setting_value is not None
+        self.spin.set_value(float(self.setting_value))
         self.spin.set_halign(Gtk.Align.FILL)
         self.spin.set_valign(Gtk.Align.CENTER)
         self.spin.connect('notify::value', self.on_value_change)
@@ -505,16 +593,16 @@ class SpinSetting(GenericSetting):
 
         self.show_all()
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         self.spin.grab_focus()
 
-    def on_value_change(self, spin, *args):
+    def on_value_change(self, spin: Gtk.SpinButton, *args: Any) -> None:
         value = spin.get_value_as_int()
         self.set_value(value)
 
 
 class FileChooserSetting(GenericSetting):
-    def __init__(self, *args, filefilter):
+    def __init__(self, *args: Any, filefilter: tuple[str, str]) -> None:
         GenericSetting.__init__(self, *args)
 
         button = Gtk.FileChooserButton(title=self.label,
@@ -523,8 +611,10 @@ class FileChooserSetting(GenericSetting):
 
         # GTK Bug: The FileChooserButton expands without limit
         # get the label and use set_max_wide_chars()
-        label = button.get_children()[0].get_children()[0].get_children()[1]
-        label.set_max_width_chars(20)
+        inner_button = cast(Gtk.Button, button.get_children()[0])
+        inner_box = cast(Gtk.Box, inner_button.get_children()[0])
+        inner_label = cast(Gtk.Label, inner_box.get_children()[1])
+        inner_label.set_max_width_chars(20)
 
         if filefilter:
             name, pattern = filefilter
@@ -540,6 +630,7 @@ class FileChooserSetting(GenericSetting):
         button.add_filter(filter_)
 
         if self.setting_value:
+            assert isinstance(self.setting_value, str)
             button.set_filename(self.setting_value)
         button.connect('selection-changed', self.on_select)
 
@@ -551,53 +642,67 @@ class FileChooserSetting(GenericSetting):
 
         self.show_all()
 
-    def on_select(self, filechooser):
+    def on_select(self, filechooser: Gtk.FileChooser) -> None:
         self.set_value(filechooser.get_filename() or '')
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         pass
 
 
 class CallbackSetting(GenericSetting):
-    def __init__(self, *args, callback):
+    def __init__(self, *args: Any, callback: Callable[..., Any]) -> None:
         GenericSetting.__init__(self, *args)
         self.callback = callback
         self.show_all()
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         self.callback()
 
 
 class ActionSetting(GenericSetting):
-    def __init__(self, *args, account):
+    def __init__(self, *args: Any, account: str) -> None:
         GenericSetting.__init__(self, *args)
-        action_name = '%s%s' % (account, self.value)
-        self.action = gtkgui_helpers.get_action(action_name)
+        action_name = f'{account}{self.value}'
+        self.action = app.app.lookup_action(action_name)
+        if self.action is None:
+            log.error('Action not found: %s', action_name)
+            return
         self.variant = GLib.Variant.new_string(account)
         self.on_enable()
 
         self.show_all()
-        self.action.connect('notify::enabled', self.on_enable)
+        self._handler_id = self.action.connect(
+            'notify::enabled', self.on_enable)
+        self.connect('destroy', self._on_destroy)
 
-    def on_enable(self, *args):
-        self.set_sensitive(self.action.get_enabled())
+    def _on_destroy(self, *args: Any) -> None:
+        if self.action is not None:
+            self.action.disconnect(self._handler_id)
 
-    def on_row_activated(self):
-        self.action.activate(self.variant)
+    def on_enable(self, *args: Any) -> None:
+        if self.action is not None:
+            self.set_sensitive(self.action.get_enabled())
+
+    def on_row_activated(self) -> None:
+        if self.action is not None:
+            self.action.activate(self.variant)
 
 
 class LoginSetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
-        self.setting_value.set_selectable(True)
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         jid = app.get_jid_from_account(self.account)
         return jid
 
 
 class PopoverSetting(GenericSetting):
-    def __init__(self, *args, entries, **kwargs):
+    def __init__(self,
+                 *args: Any,
+                 entries: Union[list[str], dict[str, str]],
+                 **kwargs: Any
+                 ) -> None:
         GenericSetting.__init__(self, *args)
 
         self._entries = self._convert_to_dict(entries)
@@ -607,7 +712,7 @@ class PopoverSetting(GenericSetting):
         box.set_halign(Gtk.Align.END)
         box.set_hexpand(True)
 
-        self._default_text = kwargs.get('default-text')
+        self._default_text = cast(str, kwargs.get('default-text'))
 
         self._current_label = Gtk.Label()
         self._current_label.set_valign(Gtk.Align.CENTER)
@@ -646,6 +751,7 @@ class PopoverSetting(GenericSetting):
         text = self._entries.get(self.setting_value, self._default_text or '')
         self._current_label.set_text(text)
 
+        assert isinstance(self.value, str)
         app.settings.connect_signal(self.value,
                                     self._on_setting_changed,
                                     account=self.account,
@@ -656,19 +762,20 @@ class PopoverSetting(GenericSetting):
         self.show_all()
 
     @staticmethod
-    def _convert_to_dict(entries):
+    def _convert_to_dict(entries: Union[list[Any], dict[Any, Any]]
+                         ) -> dict[Any, Any]:
         if isinstance(entries, list):
             entries = {key: key for key in entries}
         return entries
 
-    def _on_setting_changed(self, value, *args):
+    def _on_setting_changed(self, value: str, *args: Any) -> None:
         text = self._entries.get(value)
         if text is None:
             text = self._default_text or ''
 
         self._current_label.set_text(text)
 
-    def _add_menu_entries(self):
+    def _add_menu_entries(self) -> None:
         if self._default_text is not None:
             self._menu_listbox.add(PopoverRow(self._default_text, ''))
 
@@ -677,37 +784,44 @@ class PopoverSetting(GenericSetting):
 
         self._menu_listbox.show_all()
 
-    def _on_menu_row_activated(self, listbox, row):
+    def _on_menu_row_activated(self,
+                               listbox: Gtk.ListBox,
+                               row: PopoverRow
+                               ) -> None:
         listbox.unselect_all()
         self._popover.popdown()
 
         self.set_value(row.value)
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         self._popover.popup()
 
-    def update_entries(self, entries):
+    def update_entries(self, entries: Union[list[str], dict[str, str]]) -> None:
         self._entries = self._convert_to_dict(entries)
         self._menu_listbox.foreach(self._menu_listbox.remove)
         self._add_menu_entries()
 
-    def _on_destroy(self, *args):
+    def _on_destroy(self, *args: Any) -> None:
+        self._popover.destroy()
         app.settings.disconnect_signals(self)
 
 
 class PopoverRow(Gtk.ListBoxRow):
-    def __init__(self, label, value):
+    def __init__(self, label: str, value: str) -> None:
         Gtk.ListBoxRow.__init__(self)
         self.label = label
         self.value = value
 
-        label = Gtk.Label(label=label)
-        label.set_xalign(0)
-        self.add(label)
+        row_label = Gtk.Label(label=label)
+        row_label.set_xalign(0)
+        self.add(row_label)
 
 
 class ComboSetting(GenericSetting):
-    def __init__(self, *args, combo_items):
+    def __init__(self,
+                 *args: Any,
+                 combo_items: list[Union[str, tuple[str, str]]]
+                 ) -> None:
         GenericSetting.__init__(self, *args)
 
         self.combo = MaxWidthComboBoxText()
@@ -727,18 +841,20 @@ class ComboSetting(GenericSetting):
         self.setting_box.pack_start(self.combo, True, True, 0)
         self.show_all()
 
-    def on_value_change(self, combo):
-        self.set_value(combo.get_active_id())
+    def on_value_change(self, combo: Gtk.ComboBox) -> None:
+        active_id = combo.get_active_id()
+        if active_id is not None:
+            self.set_value(active_id)
 
-    def on_row_activated(self):
+    def on_row_activated(self) -> None:
         pass
 
 
 class PrioritySetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         adjust = app.settings.get_account_setting(
             self.account, 'adjust_priority_with_status')
         if adjust:
@@ -749,24 +865,24 @@ class PrioritySetting(DialogSetting):
 
 
 class CutstomHostnameSetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         custom = app.settings.get_account_setting(self.account,
                                                   'use_custom_host')
         return Q_('?switch:On') if custom else Q_('?switch:Off')
 
 
 class ChangePasswordSetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
 
-    def show_dialog(self, parent):
+    def show_dialog(self, parent: Gtk.Window) -> None:
         parent.destroy()
         open_window('ChangePassword', account=self.account)
 
-    def update_activatable(self):
+    def update_activatable(self) -> None:
         activatable = False
         if self.account in app.connections:
             con = app.connections[self.account]
@@ -776,36 +892,36 @@ class ChangePasswordSetting(DialogSetting):
 
 
 class CutstomAutoAwaySetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         value = app.settings.get('autoaway')
         return Q_('?switch:On') if value else Q_('?switch:Off')
 
 
 class CutstomAutoExtendedAwaySetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         value = app.settings.get('autoxa')
         return Q_('?switch:On') if value else Q_('?switch:Off')
 
 
 class CustomStunServerSetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         value = app.settings.get('use_stun_server')
         return Q_('?switch:On') if value else Q_('?switch:Off')
 
 
 class NotificationsSetting(DialogSetting):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         DialogSetting.__init__(self, *args, **kwargs)
 
-    def get_setting_value(self):
+    def get_setting_value(self) -> str:
         value = app.settings.get('show_notifications')
         return Q_('?switch:On') if value else Q_('?switch:Off')
