@@ -12,32 +12,18 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
 
-'''
-Base class for implementing plugin.
+from __future__ import annotations
 
-:author: Mateusz Biliński <mateusz@bilinski.it>
-:since: 1st June 2008
-:copyright: Copyright (2008) Mateusz Biliński <mateusz@bilinski.it>
-:license: GPL
-'''
-
-from typing import List  # pylint: disable=W0611
-from typing import Tuple  # pylint: disable=W0611
-from typing import Dict  # pylint: disable=W0611
-from typing import Any  # pylint: disable=W0611
+from typing import Any
 
 import os
-import locale
 import logging
 import pickle
 
 from gajim.common import configpaths
-from gajim.common.types import PluginExtensionPoints  # pylint: disable=W0611
-from gajim.common.types import EventHandlersDict  # pylint: disable=W0611
-from gajim.common.types import PluginEvents  # pylint: disable=W0611
-
-from gajim.plugins.helpers import log
-from gajim.plugins.gui import GajimPluginConfigDialog
+from gajim.common.events import ApplicationEvent
+from gajim.common.ged import HandlerFuncT
+from gajim.common.types import PluginExtensionPoints
 
 
 log = logging.getLogger('gajim.p.plugin')
@@ -47,74 +33,13 @@ class GajimPlugin:
     '''
     Base class for implementing Gajim plugins.
     '''
-    name = ''
-    '''
-    Name of plugin.
 
-    Will be shown in plugins management GUI.
+    __path__: str = ''
 
-    :type: str
-    '''
-    short_name = ''
-    '''
-    Short name of plugin.
+    encryption_name: str = ''
+    manifest: Any
 
-    Used for quick identification of plugin.
-
-    :type: str
-
-    :todo: decide whether we really need this one, because class name (with
-            module name) can act as such short name
-    '''
-    encryption_name = ''
-    '''
-    Name of the encryption scheme.
-
-    The name that Gajim displays in the encryption menu.
-    Leave empty if the plugin is not an encryption plugin.
-
-    :type: str
-
-    '''
-    version = ''
-    '''
-    Version of plugin.
-
-    :type: str
-
-    :todo: decide how to compare version between each other (which one
-            is higher). Also rethink: do we really need to compare versions
-            of plugins between each other? This would be only useful if we detect
-            same plugin class but with different version and we want only the newest
-            one to be active - is such policy good?
-    '''
-    description = ''
-    '''
-    Plugin description.
-
-    :type: str
-
-    :todo: should be allow rich text here (like HTML or reStructuredText)?
-    '''
-    authors = []  # type: List[str]
-    '''
-    Plugin authors.
-
-    :type: [] of str
-
-    :todo: should we decide on any particular format of author strings?
-            Especially: should we force format of giving author's e-mail?
-    '''
-    homepage = ''
-    '''
-    URL to plug-in's homepage.
-
-    :type: str
-
-    :todo: should we check whether provided string is valid URI? (Maybe
-    using 'property')
-    '''
-    gui_extension_points = {}  # type: PluginExtensionPoints
+    gui_extension_points: PluginExtensionPoints = {}
     '''
     Extension points that plugin wants to connect with and handlers to be used.
 
@@ -128,7 +53,7 @@ class GajimPlugin:
     or when extpoint is destroyed and plugin is activate (eg. chat window
     closed).
     '''
-    config_default_values = {}  # type: Dict[str, Tuple[Any, str]]
+    config_default_values: dict[str, tuple[Any, str]] = {}
     '''
     Default values for keys that should be stored in plug-in config.
 
@@ -139,10 +64,8 @@ class GajimPlugin:
     be anything (this is the advantage of using shelve/pickle instead of
     custom-made     config I/O handling); the second one should be str (gettext
     can be used if need and/or translation is planned).
-
-    :type: {} of 2-element tuples
     '''
-    events_handlers = {}  # type: EventHandlersDict
+    events_handlers: dict[str, tuple[int, HandlerFuncT]] = {}
     '''
     Dictionary with events handlers.
 
@@ -150,30 +73,20 @@ class GajimPlugin:
     priority as first element and reference to handler function as second
     element. Priority is integer. See `ged` module for predefined priorities
     like `ged.PRECORE`, `ged.CORE` or `ged.POSTCORE`.
-
-    :type: {} with 2-element tuples
     '''
-    events = []  # type: PluginEvents
+    events: list[ApplicationEvent] = []
     '''
     New network event classes to be registered in Network Events Controller.
 
-    :type: [] of `nec.NetworkIncomingEvent` or `nec.NetworkOutgoingEvent`
-    subclasses.
     '''
 
     def __init__(self) -> None:
         self.config = GajimPluginConfig(self)
-        '''
-        Plug-in configuration dictionary.
-
-        Automatically saved and loaded and plug-in (un)load.
-
-        :type: `plugins.plugin.GajimPluginConfig`
-        '''
         self.activatable = True
+        self.active = False
         self.available_text = ''
         self.load_config()
-        self.config_dialog = GajimPluginConfigDialog(self)
+        self.config_dialog = None
         self.init()
 
     def save_config(self) -> None:
@@ -182,59 +95,55 @@ class GajimPlugin:
     def load_config(self) -> None:
         self.config.load()
 
-    def __eq__(self, plugin):
-        if self.short_name == plugin.short_name:
-            return True
+    def __eq__(self, plugin: Any) -> bool:
+        if not isinstance(plugin, GajimPlugin):
+            return False
+        return self.manifest.short_name == plugin.manifest.short_name
 
-        return False
+    def __ne__(self, plugin: Any) -> bool:
+        return self.manifest.short_name != plugin.manifest.short_name
 
-    def __ne__(self, plugin):
-        if self.short_name != plugin.short_name:
-            return True
-
-        return False
-
-    def local_file_path(self, file_name):
+    def local_file_path(self, file_name: str) -> str:
         return os.path.join(self.__path__, file_name)
 
-    def init(self):
+    def init(self) -> None:
         pass
 
-    def activate(self):
+    def activate(self) -> None:
         pass
 
-    def deactivate(self):
+    def deactivate(self) -> None:
         pass
 
 
-class GajimPluginConfig():
-    def __init__(self, plugin):
+class GajimPluginConfig:
+    def __init__(self, plugin: GajimPlugin) -> None:
         self.plugin = plugin
         self.FILE_PATH = (configpaths.get('PLUGINS_CONFIG_DIR') /
-                          self.plugin.short_name)
-        self.data = {}
+                          self.plugin.manifest.short_name)
+        self.data: dict[str, Any] = {}
 
-    def __getitem__(self, key):
-        if not key in self.data:
+    def __getitem__(self, key: str) -> None:
+        if key not in self.data:
             self.data[key] = self.plugin.config_default_values[key][0]
             self.save()
 
         return self.data[key]
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> None:
         self.data[key] = value
         self.save()
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         del self.data[key]
         self.save()
 
-    def __contains__(self, key):
+    def __contains__(self, key: str) -> bool:
         return key in self.data
 
     def __iter__(self):
-        for k in self.data.keys():
-            yield k
+        for key in self.data.keys():
+            yield key
 
     def keys(self):
         return self.data.keys()
@@ -242,11 +151,11 @@ class GajimPluginConfig():
     def items(self):
         return self.data.items()
 
-    def save(self):
+    def save(self) -> None:
         with open(self.FILE_PATH, 'wb') as fd:
             pickle.dump(self.data, fd)
 
-    def load(self):
+    def load(self) -> None:
         if not self.FILE_PATH.is_file():
             self.data = {}
             self.save()
@@ -257,23 +166,23 @@ class GajimPluginConfig():
             except Exception:
                 try:
                     import shelve
-                    s = shelve.open(self.FILE_PATH)
-                    for (k, v) in s.items():
-                        self.data[k] = v
+                    shelf = shelve.open(str(self.FILE_PATH))
+                    for (key, value) in shelf.items():
+                        self.data[key] = value
                     if not isinstance(self.data, dict):
                         raise GajimPluginException
-                    s.close()
+                    shelf.close()
                     self.save()
                 except Exception:
-                    enc = locale.getpreferredencoding()
-                    filename = self.FILE_PATH.decode(enc) + '.bak'
+                    filepath_bak = self.FILE_PATH.with_suffix('bak')
                     log.warning(
                         '%s plugin config file not readable. Saving it as '
                         '%s and creating a new one',
-                        self.plugin.short_name, filename)
-                    if os.path.exists(self.FILE_PATH + '.bak'):
-                        os.remove(self.FILE_PATH + '.bak')
-                    os.rename(self.FILE_PATH, self.FILE_PATH + '.bak')
+                        self.plugin.manifest.short_name, filepath_bak)
+                    if filepath_bak.exists():
+                        filepath_bak.unlink()
+
+                    self.FILE_PATH.rename(f'{self.FILE_PATH}.bak')
                     self.data = {}
                     self.save()
 
@@ -281,5 +190,6 @@ class GajimPluginConfig():
 class GajimPluginException(Exception):
     pass
 
+
 class GajimPluginInitError(GajimPluginException):
     pass
